//---------------------------------------------------
//
// Author: John Moss
// Title:  Ultrasonic sensor test program
// Date:   Apr. 18, 2007
// Files:  ultra.cpp, ultra.h
//
// Equipment: Ultrasonic Range Finder
//              ->Devantech SRF02
//
//            USB Serial Interface Connector
//				->Acroname S19-USB-SERIAL-INT-CONN
//
//---------------------------------------------------


HANDLE comm;	// global handle for comm port

void zerobuffer(char * buf)  {
	int i;
	for (i=0;i<sizeof(buf); i++)
		buf[i] = 0;
	return;
}
//------------------------------------------------------------------------

void gotoxy(int xpos, int ypos){
  COORD scrn;    
  HANDLE hOuput = GetStdHandle(STD_OUTPUT_HANDLE);
  scrn.X = xpos; scrn.Y = ypos;
  SetConsoleCursorPosition(hOuput,scrn);
}
//------------------------------------------------------------------------

int SetupComm(HANDLE & comm, char comm_port_name[]){
	DCB dcb;
	//----- Setup Port ---------------------------------------------
	comm = CreateFile(comm_port_name, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,0);
	if (comm == INVALID_HANDLE_VALUE) return 0;

	if (!GetCommState(comm, &dcb)) return 0;

	dcb.BaudRate = 9600;
	dcb.ByteSize = 8;
	dcb.Parity   = NOPARITY;
	dcb.StopBits = ONESTOPBIT;

	if (!SetCommState(comm, &dcb)) return 0;

	//----- Set Timeouts ---------------------------------------------
	COMMTIMEOUTS timeouts;	

	// always return immediately
	timeouts.ReadIntervalTimeout = MAXDWORD;

	// set to 0 to wait forever, 1000 for 1 sec between chars.
	timeouts.ReadTotalTimeoutMultiplier   = 0;
	timeouts.ReadTotalTimeoutConstant     = 0;
	timeouts.WriteTotalTimeoutMultiplier  = 0;
	timeouts.WriteTotalTimeoutConstant    = 0;

	if (!SetCommTimeouts(comm,&timeouts)) return 0;

	return 1;
}
//------------------------------------------------------------------------

int Send_FindRange(int sensor_address=0){
	char cmd[2];
	DWORD numWrite;
	zerobuffer(cmd);
	cmd[0] = (char) sensor_address;
	cmd[1] = (char) 0x50;		// take new reading
	if (!WriteFile(comm,cmd,sizeof(cmd),&numWrite,0)) return 0;
	return 1;
}
//------------------------------------------------------------------------

int Send_GetRange(int sensor_address=0){
	char cmd[2];
	DWORD numWrite;
	zerobuffer(cmd);
	cmd[0] = (char) sensor_address;
	cmd[1] = (char) 0x5E;		// get new reading
	if (!WriteFile(comm,cmd,sizeof(cmd),&numWrite,0)) return 0;
	return 1;
}
//------------------------------------------------------------------------

int Recv_GetInches(){
	char cmd[2];
	DWORD numRead;
	int inches;
	zerobuffer(cmd);
	if (!ReadFile(comm,cmd,sizeof(cmd),&numRead,0)) return -1;
	inches = (cmd[0] * 256) + cmd[1];
	return inches;
}
//------------------------------------------------------------------------

void ShowInches(int inches){
	system("cls");
	gotoxy(10,4);
	printf("%d inches",inches);
}
//------------------------------------------------------------------------