int _stdcall MyDCC_open( char comportnumber[]);
// Opens a serial port to communicate with the DCC command station 
// and establishes the required protocol: 9600,8,0,1
//
// input:  comportnumber - str "COM1" or "COM2" or ..
// output: DCC_comm - handle to the comm port
// return 0 if error
// return 1 if open and protocol accepted.
//****************************************************************************88


void _stdcall MyDCC_close();
// releases the comm port
//************************************************************************************



int _stdcall MyDCC_motor(int id, int dir, int speed);
// A System One binary command to modify a motor on a train decoder.
// The motor cmd will be buffered in the command station and 
// continually sent every 25 mu sec onto the track until another
// DCC command is sent.
// Input: id - train's long address 0 - 255
//		  dir - new direction: 1 == forward, 0 ==reverse
//  	  speed - new speed: 0-28
//
// Return: 1 -  command station took the instruction-delivery not guaranteed
//         0 -  command station did not receive the command
//         -1 - command station had problems with id
//         -2 - direction out of range
//         -3 - speed out of range
//
//************************************************************************************


int _stdcall MyDCC_Estop(int id, int dir);
// A System One binary command to cut the power on a train motor.
// The train will stop as soon as possible.
// Choosing the DCC_motor() fucntion with speed = 0 will cause 
// the motor will brake according to the inertia programmed into the decoder.
// 
// Note: there is also a DCC_kill() function which will cut the power 
// to the entire track layout.
//************************************************************************************



int _stdcall MyDCC_function(int train_id, int &fcn_state, 
				 int fcn_id, int value);
// A System One binary command to modify one of the first 4 functions
// supported by a decoder. Usually, 0==headlight, 1==taillight or
// marslights.
//
// The function cmd will be only sent a specific number of times == DCC_SENDCNT.
//	
// Input: id         - train's long address 0 - 255
//		  fcn_state  - current state of the 5 fcns
//		  fcn_id - function id 0-4 to change
//  	  value  - new value 0 == off, 1 == on
//
// Return: fcn_state - updates state variable for the functions of this train
//		   1  - command station took the instruction-delivery not guaranteed
//         0  - command station did not receive the command
//         -1 - command station had problems with train_id
//         -2 - fcn_id out of range
//         -3 - value out of range
//************************************************************************************




int _stdcall MyDCC_kill();
// A System One binary command to cut the power on the main track. 
// Used as an emergency BRAKE for ALL trains.
// All train settings are zeroed by the hardware (DCC_motor and DCC_function).
// Must call DCC_reset to re-establish power to the main track.
//
// Return: 1 -  command station took the instruction.
//         0 -  command station did not receive the command
// ************************************************************************************


int _stdcall MyDCC_reset();
// A System One binary command to restore the power on the main track. 
// Used after the DCC_kill() is called.
// All train settings are zeroed by the hardware (DCC_motor and DCC_function).
// Must call DCC_motor() and/or DCC_function() to reset trains.
//
// Return: 1 -  command station took the instruction.
//         0 -  command station did not receive the command
// ************************************************************************************


//bool DCC_testcommands(int & testvalue);
// UNDER CONSTRUCTION
// This fcn should walk the user through a series of tests
// to check out the connections from the comm port to the cmd station,
// from the cmd station to a decoder, and basic functions on a decoder.
// **********************************************************************************


//bool DCC_command( char cmd_str[]);
// UNDER CONSTRUCTION
// This fcn would send ANY command to the cmd station.
// Issues -- should it be the binary or ascii format??
// Should it compute the cksum?
// **********************************************************************************

// *********************************************************************************
// *********************************************************************************

// END DCC_comm.h
// ***********************************************************************************


