#include <fstream>
#include "trainlist.h"
#include "Train.h"

using namespace std;


//////////////////////////////////////////////////////////////////////
//Creates a new Circular header train list with the header node using
//int max as its id
//////////////////////////////////////////////////////////////////////
TrainList::TrainList()
{
   top = new Train(INT_MAX,"",-1, -1, -1,-1, "");
   if (top==NULL)
   {
   //out of heap memory cannot create new train
   }
   top->link = top;  // circle header list
  
}



TrainList::~TrainList()
{


}


/////////////////////////////////////////////////////////////////////
//Adds a inputed train to a trainlist in order of their ids (ascending order)
//////////////////////////////////////////////////////////////////////////////
void TrainList::Add(const Train & T)
{

	Train * Q = new Train(T); //make a pointer copy of Train T
		   
    Train *prev,*p;      // used for tag along

	prev=top; // prev gets header node
	p = top->link; // p gets first real node

	while (Q->GetId() > p->GetId())
	//loop until Id is less, thus we keep the list in order by id value
	{
		prev=p;                  
		p=p-> link;
		//continue traveling through list 
	} 
				
	Q->link = p;
	prev-> link = Q;
	//add the new node to the list

}

	
////////////////////////////////////////////////////////////////////////
//Deletes a train with an inputed id from a trainlist.  
//returns 1 if successful
//		  0 if not successful
///////////////////////////////////////////////////////////////////////
int TrainList::Delete(int id) 
{


	Train *prev, *p;          // used for tag along

	prev=top;
	p=top-> link;

	while (p!=top)
	//continue through list until the last one points to top. Circular header list
	{
		if (p->GetId() == id) //if we match ids in list
		{
		    prev-> link =p -> link;  
			//set pointer of prev one to link of this train thus taking it
			//out of the list

		    delete p; //return space back to the heap
			return 1;
		}

	  prev=p; 
	  p=p->link;
	  //continue through list
	}

	return 0; //couldn't find to delete
}


//////////////////////////////////////////////////////////////////
//Clears a TrainList deleting all nodes but the list still remains.
///////////////////////////////////////////////////////////////////
void TrainList::Clear(){


   Train *p,*prev;       // for tag along

   prev=top;
   p = top->link;
 
   while (p != top) 
   {
      // Advance to next item in the list
      prev=p;
      p = p->link;
      delete prev;


   } 

}


/////////////////////////////////////////////////////////////
//Returns a pointer to a train with and inputed id number
//Returns NULL if train with specified id is not found in the list
///////////////////////////////////////////////////////////////////
Train * TrainList::Find(int id)
{


	Train *prev, *p;          // used for tag along

	prev=top;
	p=top-> link;

	while (p!=top)
	//circular header list stops when last node points to top again
	{

		if (p->GetId() == id)
			return p; 
		//if we find the id return the pointer to p

	  prev=p;
	  p=p->link;
	  //continue through the list

	}

	return NULL; //can't find train with that id and went through whole list


}

