//*********************************************************************//

#include <iostream.h>
#include <conio.h>
#include <cstring.h>
#include <fstream.h>
#include <stdio.h>

//function prototypes
int Initialize_Board (int, int, int, int, int);
int Set_Chan_On (int, int);
int Set_Chan_Off (int, int);
int Get_Channel (int, int);
int Get_Port (int);
int Set_Port (int, int);
int TO_On (int);
int TO_Off (int);
int Toggle_TO (int);
int Get_TO_Position (int);

void userInitialize_Board ();
void userSet_Chan_On ();
void userSet_Chan_Off ();
void userGet_Channel ();
void userGet_Port ();
void userSet_Port ();
void userTO_On ();
void userTO_Off ();
void userToggle_TO ();
void userGet_TO_Position ();

int const TO_output = 768;	//start port of the turnout output board
int const TO_input_A = 772;     //start port of the turnout input A board
int const TO_input_B = 776;     //start port of the turnout input B board


void Menu(char &option) {
//*********************************************************************//
//   Paints the menu screen and requests the menu option.              //
//                                                                     //
//   option - returned as the user selected option                     //
//*********************************************************************//

   clrscr();
   gotoxy(10,3);
   textcolor(LIGHTBLUE);
   cprintf("C C R   I N P U T / O U T P U T    M A I N T A I N E R\n\n\n");
   textcolor(LIGHTGRAY);

   //   Initialize the picklist options

   cout << "\n";
   cout << "                 1 - INITIALIZE board settings (I/O)\n\n";
   cout << "                 2 - TOGGLE a turnout\n";
   cout << "                 3 - Turn a TURNOUT ON\n";
   cout << "                 4 - Turn a TURNOUT OFF\n\n";
   cout << "                 5 - GET turnout POSITION\n\n\n\n";
   cout << "          0 - EXIT the system\n\n";

   cout << "          Enter an option: ";
   option = getch();
} //Menu



//***************************************************************//
//                                                               //
//                       BASE FUNCTIONS                          //
//                                                               //
//***************************************************************//

//***************************************************************
//Initailizes the board (1-9) to input and output sections based
//on the input parameters 0 and 1 for A, B, CL, and CH.
//Returns: 1 when successful and 0 when error
//***************************************************************

int Initialize_Board (int board_num, int a_val, int b_val,
		      int cl_val, int ch_val)  {

   int x;			//working value
   int ok;			//flag for success

   //check for valid I/O values
   if (((a_val != 1) && (a_val != 0)) ||
       ((b_val != 1) && (b_val != 0)) ||
      ((ch_val != 1) && (ch_val != 0)) ||
      ((cl_val != 1) && (cl_val != 0)))

      return (0);

   //store new values
   x = 1;                      	//final digit format = "1 0 0 A, CH 0 B CL"
   x = x << 3;
   x = x + a_val;     		//set a_val  - 0 0 0 0, 1 0 0 A
   x = x << 1;
   x = x + ch_val;              //set ch_val - 0 0 0 1, 0 0 A CH
   x = x << 2;
   x = x + b_val;              	//set b_val  - 0 1 0 0, A CH 0 B
   x = x << 1;
   x = x + cl_val;             	//set cl_val - 1 0 0 A, CH 0 B CL

   //put value out to port
   ok = Set_Port ((TO_output-1) + (board_num * 4), x);

   return (ok);
}

//***************************************************************
//Sets the channel of a certain port based on the input parameters
//to the 'ON' position.  In the case of the I/O Board, a 1 is off
//while a 0 is on.
//Returns 1 if successful and 0 if errors.
//***************************************************************

int Set_Chan_On(int port_num, int chan_num)  {

   int portval;			//value of the port
   int new_portval;		//new value of the port to be set
   int x = 1;			//modifier for byte value

   //verify that channel is valid
   if ((chan_num < 0) || (chan_num > 7))
      return (0);		//not a valid channel

   //modify portval
   portval = Get_Port (port_num);    //get the value of the port
   x = x << chan_num;		//shift bits left "chan_num" places
   x = ~ x;           	        //flips bits "11110111"
   new_portval = portval & x;   //turns chan_num bit to "0/on"

   //put new_portval out to port and verify it
   x = Set_Port (port_num, new_portval);

   return (x);
}

//***************************************************************
//Sets the channel of a certain port based on the input parameters
//to the 'OFF' position.  In the case of the I/O Board, a 1 is off
//while a 0 is on.
//Returns 1 if successful and 0 if errors.
//***************************************************************

int Set_Chan_Off(int port_num, int chan_num)  {

   int portval;			//value of the port
   int new_portval;		//new value of the port to be set
   int x = 1;			//modifier for byte value

   //verify that channel is valid
   if ((chan_num < 0) || (chan_num > 7))
      return (0);		//not a valid channel

   //modify portval
   portval = Get_Port (port_num);     //get port value
   x = x << chan_num;		//shift bits left "chan_num" places
   new_portval = portval | x;   //turns chan_num bit to "1/off"

   //put new_portval out to port and verify it
   x = Set_Port (port_num, new_portval);

   return (x);
}

//***************************************************************
//Gets the decimal value of a channel of a certain port based
//on the input parameters.  Manipulates this port to determine
//if it is on or off.  This module uses C++ logic in that a 1
//is considered on and a 0 is off.
//Returns 1 if 'ON' or 0 if channel is 'OFF'.
//***************************************************************

int Get_Channel(int port_num, int chan_num)  {

   int portval;			//value of the port
   int x = 1;			//modifier for byte value

   portval = Get_Port (port_num);	//get port value
   portval = portval >> chan_num;       //isolate "chan" bit
   x = portval & x;		//check chan_num bit (0 or 1)
   x = x ^ 1;      		//flip bit to be C++ logic

   return (x);            	// 1 = ON, 0 = OFF
}

//***************************************************************
//Gets the decimal value of a port located on our board.  The
//port to check is sent in through the parameter port_num.
//Returns the decimal value of the port.
//***************************************************************

int Get_Port(int port_num)  {

   int portval;			//value of the port

   portval = inp (port_num); 	//get port value
   return (portval);   		//returns the value of the port
}

//***************************************************************
//Sets the decimal value of a port located on our board.  The
//port to set is sent in through the parameter port_num and the
//value to set is passed in through portval.
//Returns 1 if succesful and 0 if errors.
//***************************************************************

int Set_Port(int port_num, int portval)  {

   int value;			//value of the port

   //store new_portval to port
   value = outp (port_num, portval);

   //verify that it was stored correctly
   if (value != portval)
      return (0);		//not correctly stored

   return (1);            	//success

}


//***************************************************************
//Turns a Turnout to the 'ON' position. The turnout to switch
//is sent in through the parameter TO_num and corresponds to
//the number on the I/O Board.
//Returns a 1 if successful and a 0 if errors.
//***************************************************************

int TO_On (int TO_num)  {

   int port_num;		//port to be set
   int chan_num;		//channel to be turned on
   int ok;			//flag for successful operation

   //calculate the port and channel
   port_num = TO_output + (TO_num / 8);
   chan_num = TO_num % 8;

   //set the appropriate channel to on
   ok = Set_Chan_On (port_num, chan_num);

   return (ok);
}


//***************************************************************
//Turns a Turnout to the 'OFF' position. The turnout to switch
//is sent in through the parameter TO_num and corresponds to
//the number on the I/O Board.
//Returns a 1 if successful and a 0 if errors.
//***************************************************************

int TO_Off (int TO_num)  {

   int port_num;		//port to be set
   int chan_num;		//channel to be turned off
   int ok;			//flag for successful operation

   //calculate the port and channel
   port_num = TO_output + (TO_num / 8);
   chan_num = TO_num % 8;

   //set the appropriate channel to off
   ok = Set_Chan_Off (port_num, chan_num);

   return (ok);
}

//***************************************************************
//Toggles a Turnout to the opposite position. The turnout to switch
//is sent in through the parameter TO_num and corresponds to
//the number on the I/O Board.
//Returns a 1 if successful and a 0 if errors.
//***************************************************************

int Toggle_TO (int TO_num)  {

   int port_num;		//port to be switched
   int chan_num;		//channel to be switched
   int x;       		//current status of channel
   int ok;			//flag for successful operation

   //calculate the port and channel
   port_num = TO_output + (TO_num / 8);
   chan_num = TO_num % 8;

   //set the appropriate channel to switch
   x = Get_Channel (port_num, chan_num);
   if (x == 1)
      ok = Set_Chan_Off (port_num, chan_num);
   else if (x == 0)
      ok = Set_Chan_On (port_num, chan_num);

   return (ok);
}



//***************************************************************
//Gets the position of a turnout.
//Returns:   position A, value = -1
//           position B, value =  1
//               middle, value =  0
//***************************************************************

int Get_TO_Position (int TO_num)  {

   int port_num;		//port to be looked at
   int chan_num;		//channel to be looked at
   int a_val;			//value of channel at position A
   int b_val;                   //value of channel at position B
   int position;

   //get the channel value for position A
   port_num = TO_input_A + (TO_num / 8);
   chan_num = TO_num % 8;
   a_val = Get_Channel (port_num, chan_num);

   //get the channel value for position B
   port_num = TO_input_B + (TO_num / 8);
   chan_num = TO_num % 8;
   b_val = Get_Channel (port_num, chan_num);

   //return position of the turnout
   if (a_val > b_val)    position = -1;	//position A
   if (a_val < b_val)    position =  1;	//position B
   if (a_val == b_val)   position =  0;	//middle

   return (position);

}





//***************************************************************//
//                                                               //
//                 USER INTERFACED FUNCTIONS                     //
//                                                               //
//        These functions prompt the user for the input          //
//        parameters needed by the base functions.  They         //
//        also display significant messages to the user.         //
//                                                               //
//***************************************************************//

void userInitailize_Board()  {

   int a_val;
   int b_val;
   int cl_val;
   int ch_val;
   int reg_val;		        //value of the control reg. of the board
   int board_num; 		//number of board from 8255 to set
   int x;                       //working value
   int ok;                      //sucess flag

   clrscr();
   cout << "\n\n   Please enter the board number to intitialize (1-9): ";
   cin >> board_num;

   //check for a valid board number
   if ((board_num < 1) || (board_num > 9))  {
      cout << "\n\n   Initailization Error!!!  Not a valid Board Number.";
      getch();
      return;
   }

   reg_val = Get_Port ((TO_output-1) + (board_num * 4));   //768,769...

   //1 = input, 0 = output
   //check CL
   cl_val = reg_val & 1;

   //check B
   x = reg_val >> 1;		//shift right one place
   b_val = x & 1;

   //check CH
   x = reg_val >> 3;		//shift right three places
   ch_val = x & 1;

   //check A
   x = reg_val >> 4;		//shift right four places
   a_val = x & 1;

   //display current port settings
   cout << "\n\n   CURRENT PORT SETTINGS   (Input = 1, Output = 0)\n\n";
   cout << "   Port A  ( 0-7 ): " << a_val << endl;
   cout << "   Port B  ( 8-15): " << b_val << endl;
   cout << "   Port CL (16-19): " << cl_val << endl;
   cout << "   Port CH (20-24): " << ch_val << endl;

   //prompt for new settings (1 or 0)
   cout << "\n   Please enter new port settings (1 or 0) for...\n\n";
   cout << "   Port A  ( 0-7 ): "; cin >> a_val;
   cout << "   Port B  ( 8-15): "; cin >> b_val;
   cout << "   Port CL (16-19): "; cin >> cl_val;
   cout << "   Port CH (20-24): "; cin >> ch_val;

   //set new port settings and verify results
   ok = Initialize_Board (board_num, a_val, b_val, cl_val, ch_val);

   if (ok == 1)  {
	cout << "\n   Initailization Complete.";
	getch();
   }
   else  {
	cout << "\n   Initailization Error!!!";
	getch();
   }
   return;
}

//***************************************************************//

void userSet_Chan_On()  {

   int chan_num;    //channel (0-7)
   int port_num;    //port number (768, 769,...)
   int ok;

   clrscr();
   cout << "\n       TURN A CHANNEL 'ON'\n\n";
   cout << "   Please enter the port number: "; cin >> port_num;
   cout << "\n   Please enter the channel number (0-7): "; cin >> chan_num;

   ok = Set_Chan_On(port_num, chan_num);

   if (ok != 1)  {                         //error in processing
	cout << "\n   Error!!!";
	getch();
	return;
   }

   cout << "\n\n   Channel " << chan_num << " is set to ON.";
   getch();
   return;
}

//***************************************************************

void userSet_Chan_Off()  {

   int chan_num;    //channel (0-7)
   int port_num;    //port number (768, 769,...)
   int ok;

   clrscr();
   cout << "\n       TURN A CHANNEL 'OFF'\n\n";
   cout << "   Please enter the port number: "; cin >> port_num;
   cout << "\n   Please enter the channel number (0-7): "; cin >> chan_num;

   ok = Set_Chan_Off(port_num, chan_num);

   if (ok != 1)  {                         //error in processing
	cout << "\n   Error!!!";
	getch();
	return;
   }

   cout << "\n\n   Channel " << chan_num << " is set to OFF.";
   getch();
   return;
}

//***************************************************************

void userGet_Channel()  {

   int chan_num;    //channel (0-7)
   int port_num;    //port number (768, 769,...)
   int on;

   clrscr();
   cout << "\n       GET CHANNEL STATUS\n\n";
   cout << "   Please enter the port number: "; cin >> port_num;
   cout << "\n   Please enter the channel number (0-7): "; cin >> chan_num;

   on = Get_Channel(port_num, chan_num);

   if (on)  {
      cout << "\n\n    Channel number " << chan_num << " is ON.";
      getch();
   }
   else  {
      cout << "\n\n    Channel number " << chan_num << " is OFF.";
      getch();
   }

   return;
}

//***************************************************************

void userGet_Port()  {

   int port_num;    //port number (768, 769,...)
   int val;         //value of the port

   clrscr();
   cout << "\n       GET A PORT VALUE\n\n";
   cout << "   Please enter the port number: "; cin >> port_num;

   val = Get_Port(port_num);

   cout << "\n\n   The value of port " << port_num << " is " << val << ".";
   getch();

   return;
}

//***************************************************************

void userSet_Port()  {

   int x;			//success flag
   int portval;                 //port value to be set
   int port_num;    		//port number (768, 769,...)

   clrscr();
   cout << "\n       SET A PORT VALUE\n\n";
   cout << "   Please enter the port number to set: ";
   cin >> port_num;
   cout << "\n   Please enter the value to be set: ";
   cin >> portval;

   //store portval to port_num
   x = Set_Port(port_num, portval);

   //verify that it was stored correctly
   if (x != 1)  {
      cout << "\n   Error!!!";
      getch();
      return;
   }

   cout << "\n\n   The value of port " << port_num;
   cout << " was set to " << portval << ".";
   getch();
   return;
}

//***************************************************************

void userTO_On()  {

   int TO_num;    	//turnout number
   int ok;              //success flag

   clrscr();
   cout << "\n       TURN TURNOUT to ON\n\n";
   cout << "   Please enter the turnout number: "; cin >> TO_num;

   ok = TO_On (TO_num);

   //verify that it was turned to on
   if (ok != 1)  {
      cout << "\n   Error!!!";
      getch();
      return;
   }

   cout << "\n\n   Turnout number " << TO_num << " is ON.";
   getch();

   return;
}

//***************************************************************

void userTO_Off()  {

   int TO_num;    	//turnout number
   int ok;              //success flag

   clrscr();
   cout << "\n       TURN TURNOUT to OFF\n\n";
   cout << "   Please enter the turnout number: "; cin >> TO_num;

   ok = TO_Off (TO_num);

   //verify that it was turned to off
   if (ok != 1)  {
      cout << "\n   Error!!!";
      getch();
      return;
   }

   cout << "\n\n   Turnout number " << TO_num << " is OFF.";
   getch();

   return;
}

//***************************************************************

void userToggle_TO()  {

   int TO_num;    	//turnout number
   int ok;              //success flag

   clrscr();
   cout << "\n       TOGGLE TURNOUT (ON/OFF)\n\n";
   cout << "   Please enter the turnout number: "; cin >> TO_num;

   ok = Toggle_TO (TO_num);

   //verify that it was switched
   if (ok != 1)  {
      cout << "\n   Error!!!";
      getch();
      return;
   }

   cout << "\n\n   Turnout number " << TO_num << " is switched.";
   getch();

   return;
}


//***************************************************************

void userGet_TO_Position()  {

   int TO_num;      //turnout number (0-23)
   int position;    //position of the turnout (-1, 0, 1)

   clrscr();
   cout << "\n       GET A TURNOUT POSITION\n\n";
   cout << "   Please enter the turnout number: "; cin >> TO_num;

   position = Get_TO_Position(TO_num);

   if (position < 0)
      cout << "\n\n   Turnout " << TO_num << " is turned to position A.";
   else if (position > 0)
      cout << "\n\n   Turnout " << TO_num << " is turned to position B.";
   else
      cout << "\n\n   Turnout " << TO_num << " is in middle position.";

   getch();
   return;
}

//***************************************************************

void main() {
//********************************************************************//
//                                                                    //
//              M   A  I  N      P  R  O  G  R  A  M                  //
//                                                                    //
//********************************************************************//

   char option;
   int  quit;

   do {
      //   Paint the menu screen and input next option
      Menu(option);

      switch (option) {
      case '1':
	 userInitailize_Board();
	 break;
      case '2':
	 userToggle_TO();
	 break;
      case '3':
	 userTO_On();
	 break;
      case '4':
	 userTO_Off();
	 break;
      case '5':
	 userGet_TO_Position();
	 break;


      case '0': // EXIT
	 return;
      }// switch

   } while (1);   //forever

}//main
