/*****************************************/
/* TOS DCC Manager                       */
/*                                       */
/* Chris Kratz , SNC , 2007              */
/*****************************************/

/* Some DCC functions taken from dcc_comm.cpp, part of the original tos */
/* dcc_comm.cpp Authors: D C Pankratz, Jeremy Vosters, Created: April 27, 2001  */

#include<windows.h>
#include<iostream>
#include<fstream>
#include<stdio.h>
//#include"stdafx.h"

#include<time.h>
#include<string.h>


#include"../TOS CLIENT API/track.h"
#include"../TOS CLIENT API/KMP_CLIENT_API.H"
#include"../TOS CLIENT API/TOS_API.h"


const char DCCPORT[] = "COM1";	//Default it to com 1
const char LOGFILE[13] = "DCCMan.log";
const char CLIENTNAME[9] = "DCCMan";

const int CYCLESLEEP =  2;
const int MAXAIU = 10;
const int MAXUSERS = 20;
const int COMDELAY = 1;

bool InitializeComm(HANDLE& dcc, char * port, unsigned long length);
bool SetTrainSpeed(HANDLE commptr, char * message, unsigned long length);
bool ReadAIU(HANDLE dcc, char * data, unsigned long length, bool * value, bool& single);
bool ReadAIU(HANDLE dcc, unsigned long add, bool * value);
void BuildAIUList(Track * myTrack, unsigned long * list, unsigned long& cnt);
bool UpdateSensorData(HANDLE dcc, HANDLE pipe, Track * myTrack, unsigned long * list, unsigned cnt, char users[][CLIENT_SIZE_CLIENTNAME], unsigned long usercnt);
bool SetTurnout(HANDLE dcc, char * data, unsigned long length);



void WriteLog(char * message);
using namespace std;
void main(int argc, char* argv[])
{
	/* Grab the comport your using from command line*/
	//DCCPORT = argv[0];
	//cout<<" USING "<<DCCPORT;

	/* Connect to the server */
	bool DCC_State = false;

	char PipeName[255];
	strcpy(PipeName, "\\\\.\\pipe\\");
	strcat(PipeName, CLIENTNAME);


	//HANDLE pipe;
	HANDLE pipe = ConnectToServer(PipeName);

	if(pipe == INVALID_HANDLE_VALUE)
	{	
		WriteLog("Failed to connect to server.");
		return;
	}
	WriteLog("Connected To server.");

	HANDLE dcc = NULL;

	Track myTrack;

	TOS_RequestTrackData(pipe);
	TOS_RequestTrackDataUpdates(pipe);

	unsigned long AIUList[MAXAIU];
	unsigned long AIUCount = 0;

	char UpdateUsers[MAXUSERS][CLIENT_SIZE_CLIENTNAME];
	unsigned long UserCnt = 0;


	char From[CLIENT_SIZE_CLIENTNAME];
	unsigned long ServiceCode;
	char MessageData[CLIENT_SIZE_MESSAGE];
	unsigned long MessageLength;

	char logmessage[255];
	char filename[255];
	unsigned long addr = 0;

	bool critical = false;

	while(true)
	{
		int ret = -1;
		if(!critical)
		{
		/* Check Sensors */
		//if(DCC_State)
		//	UpdateSensorData(dcc, pipe, &myTrack, AIUList, AIUCount, UpdateUsers, UserCnt);


		/* Get Next Message From Queue */
		ret = GetMessageFromPeer(From, ServiceCode, MessageData, MessageLength, pipe);
		}

		if(ret >= 0)
		{
			/* DCC MAN SERVICE CODES                             */
			/* Defined In TOS_API.H								 */

			//cerr << "Received";

			switch( ServiceCode )
			{
			case SC_INIT_DCC:
				if(DCC_State == true)
				{
					TOS_DCCReady(From, pipe);
					break;
				}
				/* Connect to dcc box */
				if(InitializeComm(dcc, MessageData, MessageLength))
				{
					cout<< "COM CONNECTED"<<endl;
					DCC_State = true;
					TOS_DCCReady(From, pipe);
				}
				else
				{
					cout<< "COM NOT CONNECTED"<<endl;
					MessageData[MessageLength] = 0;
					sprintf(logmessage, "Failed to open DCC on %s.", MessageData);
					WriteLog(logmessage);
					TOS_DCCNotReady(From, pipe);
				}
				break;
				
			case SC_SET_TRAIN_SPEED_DCC:
				/* SEND DCC COMMAND TO SET TRAIN SPEED */
				if(!SetTrainSpeed(dcc, MessageData, MessageLength))
				{
					sprintf(logmessage, "Failed to open change train speed.");
					WriteLog(logmessage);
				}
				break;
			
			case SC_READ_AIU:
				critical = true;
				/* read aiu */
				bool value[14];
				
				addr = MessageData[0] * 256;
				addr += MessageData[1];
				//cout<<endl<<addr<<"--";
				ReadAIU(dcc, addr, value);
				//for(int j = 0; j<14; j++)
				//{
				//	cout<<value[j];
				//}

				char message[6];
				message[0] = MessageData[0];
				message[1] = MessageData[1];

				message[2] = 0;
				message[3] = 0;
				message[4] = 0;
				message[5] = 0;
				for(int j = 0; j<4; j++)
				{
					message[2] += value[j];
					message[2]<<1;
				}
				
				for(int j = 4; j<8; j++)
				{
					message[3] += value[j];
					message[3]<<1;
				}
				
				for(int j = 8; j<12; j++)
				{
					message[4]+= value[j];
					message[4]<<1;
				}
				
				for(int j = 12; j<14; j++)
				{
					message[5] += value[j];
					message[5]<<1;
				}
				message[5]<<2;

				for(int j = 0; j<6; j++)
				{
					cout<<(unsigned int)message[j]<<endl;
				}
				

				SendMessageToPeer(message, 6, SC_AIU_UPDATES, From, pipe);	
				//delete message;
				critical = false;
				break;

			case SC_TRACK_DATA:
				/* load track data */
				TOS_ReadTrackData(&myTrack, MessageData, MessageLength);
				BuildAIUList(&myTrack, AIUList, AIUCount);
				break;

			case SC_REQUEST_DCC_UPDATES:
				/* add to list */
				if(UserCnt >= MAXUSERS)
				{
					sprintf(logmessage, "User %s not added to update list.", From);
					WriteLog(logmessage);
					break;
				}
				strncpy(UpdateUsers[UserCnt], From, CLIENT_SIZE_CLIENTNAME);
				UserCnt++;
				break;

			case SC_SET_TURNOUT:
				/* message format */
				/* [dcc address(x2)][value(x1)] */
				if(!SetTurnout(dcc, MessageData, MessageLength))
				{
					WriteLog("Failed to set turnout.");
				}
				break;

			default:
				/* UNRECOGNIZED COMMAND - Do nothing */
				WriteLog("Unrecognized command.");
				break;
			}
		}
		/* Release CPU */
		Sleep(CYCLESLEEP);
	}
}

bool SetTurnout(HANDLE dcc, char * data, unsigned long length)
{
	if(length != 3)
		return false;

	unsigned long add = data[0] * 256;
	add += data[1];

	char val = data[2];

	// acc_decoder - sends command to change state of switch-it
	//	accessory decode on NCE powerhouse DCC
	//
	//		dcc		-> com port handle
	//		address	-> accesory 
	//		val		-> state to set decoder

	unsigned char command[10];

	char ret[100];	//not sure what might be returned

	// magic numbers game
	// 10AAAAAA AAAA1AAX EEEEEEEE
	unsigned char addr = add;

	unsigned int state = 0;
	if(val) state = 1;

	// this section calculates the address and state 
	// into a usable packet
	// does not appear to comply with NMRA standards
	//	for accessory decoders
	int in1 = 0;
	int in2 = 0;

	/********************************* USED FOR TESTING***************************/
	//cout<<" ENTER FIRST CHUNK: ";
	//cin>>hex>>in1;
	//cout<<endl<<" ENTER SECOND CHUNK: ";
	//cin>>hex>>in2;
	//cout<<endl;
	//unsigned char b1 = in1;				//address
	//unsigned char b2 = in2;		//address and state

	//revised magic numnbers game:	ex) a6=6th bit of address
	//Byte 1: 1.0.a7.a6.a5.a4.a3.a2	--address byte shifted 2 to the right and a 10 at the front
	//Byte 2: 1.1.1.1.1.a1.a0.s		--(4) 1's and the C (activate bit), the last two addresses and the State(s) bit
	//Byte 3: e.e.e.e.e.e.e.e		--Check sum
	//There is very little documentation online for this stuff.....
	/****************************************************************************/

	unsigned char b1 = 0x80 + ((addr-1)>>2) +1;				//address (shifts two to the right)
	unsigned char newadd = (addr-1) & 0x03;			//bit mask to get first to address bits
	unsigned char b2 = 0xF0 + 8 + (newadd<<1) + state;		//address and state
	unsigned char b3 = b1 ^ b2;							//checksum

	//char buffer[100];
	command[0] = 0x93;	// NCE command (3 byte packet)
	command[1] = 0x0A;	// repeat ten times
	command[2] = b1;	// first byte of packet
	command[3] = b2;
	command[4] = b3;
	
	//cout<<"B1 " <<(addr/4) + 1<< " b2 " << 8 + (addr%2) <<" state "<< state<<endl;

	cout<<(int)command[0]<<endl;
	cout<<(int)command[1]<<endl;
	cout<<(int)command[2]<<endl;
	cout<<(int)command[3]<<endl;
	cout<<(int)command[4]<<endl;
	/* Write Commmand to DCC station */
	DWORD numWrite;
	int success = WriteFile(dcc ,command,5,&numWrite,0);
	
	
    if (!success)
		return false;

	char buffer[4];

	/* WAIT FOR '!' acknowlege from DCC */
	DWORD numRead;
	unsigned long j;
	for(j=0;j<50;j++) /* retry 50 times */
	{
		success = ReadFile(dcc,buffer,1,&numRead,0);
		if (!success)
			return false;

		if(numRead >= 1)
			break;
		
		Sleep(COMDELAY);
	}
	
	/* done */
	return true;

}

void BuildAIUList(Track * myTrack, unsigned long * list, unsigned long& cnt)
{
	//THIS IS NOT NEEDED IF YOU ARE NOT USING THE TrackMan and TrainMan exe's
	unsigned long scnt = myTrack->GetSensorCount();
	cnt = 0;
	unsigned long i,j;
	for(i=0;i<scnt;i++)
	{
		bool found = false;
		for(j=0;j<cnt;j++)
		{
			if(myTrack->GetSensorByIndex(i).AIU == list[j])
			{
				found = true;
				break;
			}
		}
		if(!found)
		{
			list[cnt] = myTrack->GetSensorByIndex(i).AIU;
			cnt++;
		}
	}

}
				


bool UpdateSensorData(HANDLE dcc, HANDLE pipe, Track * myTrack, unsigned long * list, unsigned cnt, char  users[][CLIENT_SIZE_CLIENTNAME], unsigned long usercnt)
{
	//THIS IS NOT NEEDED IF YOU ARE NOT USING THE TrackMan and TrainMan exe's
	unsigned long scnt = myTrack->GetSensorCount();

	unsigned int i,j;
	for(i=0;i<cnt;i++)
	{
		bool value[14];
		if(ReadAIU(dcc, list[i], value))
		{
			unsigned int AIU = list[i];
			for(j=0;j<scnt;j++)
			{
				Sensor s = myTrack->GetSensorByIndex(j);
				if(s.AIU == list[i])
				{
					if(myTrack->SetSensorValue(j, value[s.pin-1]))
					{
						/*change in data*/
						cerr << "Sending update.";
						TOS_SendAIUUpdate(j, value[s.pin-1], users, usercnt, pipe);
					}
				}
			}
		}
		else
		{
			WriteLog("AIU Get Failed.");
		}
	}
	return true;

}

bool ReadAIU(HANDLE dcc, unsigned long add, bool * value)
{
	//THIS HANDLES ALL OF THE TRACK STATUS BACK TO THE INTERFACE
	char command[3];
	//Command to send to AIU to read it
	command[0] = 0x8A;	//command 
	command[1] = add;		// Cab Address
	command[2] = '\0';	// writecomm uses strlen to determine # bytes to send

	//hcomm is object of class track 
	// handle to com port file
	
	DWORD numWrite;
	int success = WriteFile(dcc ,command,strlen(command),&numWrite,0);
    if (!success)
		return false;
	//cout<<"Sending get from AIU cmd"<<endl
		;
	char buffer[8];
	for(int i = 0; i < 8; i++)
	{
		buffer[i] = ' ';
	}

	//Sleep(COMDELAY);

	DWORD numRead;
	unsigned long totalRead = 0;
	unsigned long j;
	for(j=0;j<50;j++) /* retry 50 times */
	{
		success = ReadFile(dcc,&buffer[totalRead],4,&numRead,0);
		if (!success)
			return false;
		//cout<<"Receving AIU status:"<<j<<endl;
		totalRead += numRead;

		if(totalRead >= 4)
			break;
		
		Sleep(COMDELAY);
	}
	
	

	if( totalRead != 4){
		//incorrcect number of bytes read from com
		//cerr << "NOT ENOUGH BYTES READ!\n";
		return false;
	}

	/* buffer contains data and sticks it all back into the 'value' var to be sent back to user*/
	unsigned long i;
	for(i=0;i<6;i++)
	{
		char test = buffer[0];
		test = test >> i;
		value[i+8] = !(test & 0x01);
	}
	for(i=0;i<8;i++)
	{
		char test = buffer[1];
		test = test >> i;
		value[i] = !(test & 0x01);
	}
	for(i = 0; i<14; i++)
	{
		cout<<value[i];
	}

	cout<<endl;

	return true;
}

bool ReadAIU(HANDLE dcc, char * data, unsigned long length, bool * value, bool& single)
{
	/*message format */
	/* [address(x2)][input(x1)] */
	//&&&&&&&&&&&&&&&&&&&&&&&&&&& THIS WAS IMPLEMENTED OUT OF THE STATE DETERMINATION LOOP&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

	if(value = NULL)
		return false;
	
	unsigned long add = data[0] * 256;
	add += data[1];
	unsigned char input = data[3];

	if(input > 14 || input < 1)
		return false;

	if(! ReadAIU(dcc, add, value))
		return false;

	single = value[input];

	return true;


}

bool SetTrainSpeed(HANDLE commptr, char * message, unsigned long length) 
{
	/* FROM dcc_comm.cpp */
//WORKS WELL AS OF 5/6/09
// A System One binary command to modify a motor on a train decoder.
// The motor cmd will be buffered in the command station and 
// continually sent every 25 mu sec onto the track until another
// DCC command is sent.
// Input: id - train's long address 0 - 255
//		  dir - new direction: 1 == forward, 0 ==reverse
//  	  #speed - new speed: 0-28#
//		  spee - new speed: 0-14
//
// Return: 1 -  command station took the instruction-delivery not guaranteed
//         0 -  command station did not receive the command
//         -1 - command station had problems with id
//         -2 - direction out of range
//         -3 - speed out of range
//************************************************************************************

		/* message format */
	/* [DCC address (x2)][Speed(x2)][direction(x1)] */
	if (length != 5)
	{
		return false;
	}

	unsigned long id  = message[0] * 256;
	id += message[1];
	unsigned long speed = message[2]  * 256;
	speed += message[3];
	char dir = message[4];

	char cmd[10], c;
	memset(cmd, 0, 10);

	cmd[0] = (char) 0xA3; // opcode for 3 byte packet
	cmd[1] = (char) id;
	
	/* old stuff, works but not sure if it is right
	if (speed < 0 || speed > 28) return false;// return -3; 
	if (speed <= 1 || speed <= 14) speed++;  // DCC reserved 0 = brake and 1 estop
	else if ( speed >= 15)  speed += 3; // DCC reserved 16 = brake and 17 = estop
	// 0 == 16: brake(chip can be prototyped for inertia)
	// 1 == 17: emergency stop - cut the motor. 
	// mode byte: 01dsssss where d = dir and s = speed
	*/
	if(speed > 13)
		speed = 13;


	speed+=2;

	
	if ( dir == 0 || dir == 1) {
		dir = dir << 5; // shift left 5
		cmd[2] = 0x40 | (char)dir | speed; // add in the direction and speed
	}
	/*
	if(dir)
		cmd[2] = 0x40;
	else
		cmd[2] = 0x60;

	cmd[2] = cmd[2] | speed;
	*/
	

	cmd[3] = cmd[1] ^ cmd[2]; // chksum is the xor

	DWORD numWrite;
	int success = WriteFile(commptr,cmd,strlen(cmd),&numWrite,0);
    if (!success)
		return false;

	/* read */
	char response[255];

	Sleep(COMDELAY);

	DWORD numRead;
	success = ReadFile(commptr,response,1,&numRead,0);
	if (!success)
		return false;

	if(numRead == 0)
		return false;

	if (response[0]=='1') return false;//return -1;

	return true; //return 1;

} // DCC_motor



bool InitializeComm(HANDLE& dcc, char * port, unsigned long length)
{
	//
	/* from original dcc_comm.cpp file */
	port[length] = 0;
	

	dcc = CreateFile(port, GENERIC_READ | GENERIC_WRITE,
			   0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,0);

	//You actually can use something besides the default port
	cout<<"USING "<< port << " " << dcc <<endl;
	
	if (dcc == INVALID_HANDLE_VALUE) return false;
	cout<<"DCC handle: "<<dcc<<endl;


	/* Setup com port */
	DCB dcb;

	COMMTIMEOUTS timeouts;
    BOOL fsuccess;

    fsuccess = GetCommState(dcc, &dcb);

	if (!fsuccess) return false;
	cout<<"Success handle: "<<fsuccess<<endl;
	dcb.BaudRate = 9600;
	dcb.ByteSize = 8;
	dcb.Parity   = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	fsuccess = SetCommState(dcc, &dcb);
	if (!fsuccess) return false;
	cout<<"Success handle2: "<<fsuccess<<endl;
		
	// Set the readintervaltimeout so that ReadFile returns
	// immediately even if the comm buffer is empty.
	timeouts.ReadIntervalTimeout = MAXDWORD; //immediate read
	// set to 0 to wait forever, 1000 for 1 sec between chars.
    timeouts.ReadTotalTimeoutMultiplier  = 0;
    timeouts.ReadTotalTimeoutConstant    = 0;
	timeouts.WriteTotalTimeoutMultiplier = 0;
	timeouts.WriteTotalTimeoutConstant    = 0;
	fsuccess = SetCommTimeouts(dcc,&timeouts);
	
	if (!fsuccess) return false;
	cout<<"Success handle3: "<<fsuccess<<endl;
	return true;


}

void WriteLog(char * message)
{

	char LogFile[255];
	strcpy(LogFile, LOGFILE);

	ofstream log;
	log.open(LogFile,ios::app);

	cerr << message << "\n";

	if(!log)
	{
		cerr << "Failed to write to log.\n";
		return;
	}
    char dbuffer [9];
    char tbuffer [9];
    _strdate( dbuffer );
    _strtime( tbuffer );
    

	log << dbuffer << " " << tbuffer << " - " << message << "\n";

	return;

}








