#ifndef _PPLIB95_H
#define _PPLIB95_H

// pplib95.h
// Include file for applications that use PEEKPOKE driver.  Win95 Version.
// Define the IOCTL codes we will use.  The IOCTL code contains a command
// identifier, plus other information about the device, the type of access
// with which the file must have been opened, and the type of buffering.
//
// by Steven Klug
// for Industrial Computer Source
//
// $Id: pplib95.h 1.2 1997/05/23 18:02:23 sklug Exp sklug $
// $Date: 1997/05/23 18:02:23 $
// $Log: pplib95.h $
// Revision 1.2  1997/05/23 18:02:23  sklug
// Revision 1.1  1997/05/13 17:48:09  steve
// Initial Revision
//-----------------------------------------------------------------------------

// Device type           -- in the "User Defined" range."
#define PP_TYPE 41001

// Data structures
// NOTE:  You must
// compile with a maximum of 4 byte stucture alignment for this
// to function correctly.  If you are using 8 byte alignment, you must
// bracket your declaraction with a #pragma pack (push,4), #pragma pack (pop)

// Used for IOCTL_PP_WRITE_PORT_... functions.
typedef struct _WRITE_PORT
{
	ULONG Port;
	ULONG Data;
} WRITE_PORT, *PWRITE_PORT;

// Used for IOCTL_PP_MAP_MEM.
typedef struct _MAP_MEM
{
	ULONG Page;		// The starting page to reserve (pages are 4096 bytes).
	ULONG Size;		// Size in pages to reserve.
					// I.e. for a0000-affff, use Page 0xa0, Size 0x10
} MAP_MEM, *PMAP_MEM;

// Used for IOCTL_PP_MAP_MEM.
typedef struct _UNMAP_MEM
{
	PVOID Address;	// The pointer returned from the MAP operation.
	ULONG Size;		// Size in pages that was reserved reserve.
} UNMAP_MEM, *PUNMAP_MEM;

// The IOCTL function codes from 0x800 to 0xFFF are for customer use.
#define IOCTL_PP_READ_PORT_UCHAR \
    CTL_CODE( PP_TYPE, 0x900, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_PP_READ_PORT_USHORT \
    CTL_CODE( PP_TYPE, 0x901, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_PP_READ_PORT_ULONG \
    CTL_CODE( PP_TYPE, 0x902, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_PP_WRITE_PORT_UCHAR \
    CTL_CODE( PP_TYPE, 0x903, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_PP_WRITE_PORT_USHORT \
    CTL_CODE( PP_TYPE, 0x904, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_PP_WRITE_PORT_ULONG \
    CTL_CODE( PP_TYPE, 0x905, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_PP_MAP_MEM \
    CTL_CODE( PP_TYPE, 0x906, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_PP_UNMAP_MEM \
    CTL_CODE( PP_TYPE, 0x907, METHOD_BUFFERED, FILE_READ_ACCESS )


// Function prototypes.
// Because a local static handle is used to access the driver, you must make a call to open the driver in the first thread of your application.   This will allow thread-safe access.
#ifdef __cplusplus
extern "C" {
#endif

// Init/shutdown functions
BOOL ics_pp_open ( void );
void ics_pp_close ( void );

// Port Output functions
int ics_pp_outp ( unsigned short port, int data );
unsigned short ics_pp_outpw ( unsigned short port, unsigned short data );
unsigned long ics_pp_outpl ( unsigned short port, unsigned long data );

// Port Input functions
int ics_pp_inp ( unsigned short port );
unsigned short ics_pp_inpw ( unsigned short port );
unsigned long ics_pp_inpl ( unsigned short port );

// Memory access functions.
// ics_pp_make_pointer returns a void pointer to the requested page
// in memory.  Pages are 4096 byte blocks of memory.  For your
// typical ISA card aperture at A000, of 64k length, specify 
// page 0xA0, length 0x10 (16 pages).  Attempts to access past the length
// specified will most likely cause the application to be terminated.
void *ics_pp_make_pointer ( int page, int length );

// ics_pp_release_pointer should be called when you're finished using
// the pointer to the requested memory region.  Failure to call this
// could leave areas in memory permanently inaccessable to Win95.
// Pass in the pointer returned from ics_pp_make_pointer as the address
// as well as the length of the block.
void ics_pp_release_pointer ( void *address, int length );

#ifdef __cplusplus
}
#endif

// Compatibility macros
#define _outp(X,Y) ics_pp_outp(X,Y)
#define _outpw(X,Y) ics_pp_outpw(X,Y)
#define _outpl(X,Y) ics_pp_outpl(X,Y)

#define _inp(X) ics_pp_inp(X)
#define _inpw(X) ics_pp_inpw(X)
#define _inpl(X) ics_pp_inpl(X)

#endif
